@@ -26,6 +26,7 @@
 #include <QObject>
 #include <QStandardPaths>
 #include <QDir>
+#include <QCoreApplication>
 
 class PQCConfigFiles {
 
@@ -43,6 +44,18 @@ public:
         return m_CONFIG_DIR;
     }
 
+    const QString EXTENSION_CONFIG_DIR() {
+        return m_EXTENSION_CONFIG_DIR;
+    }
+
+    const QString EXTENSION_DATA_DIR() {
+        return m_EXTENSION_DATA_DIR;
+    }
+
+    const QString EXTENSION_CACHE_DIR() {
+        return m_EXTENSION_CACHE_DIR;
+    }
+
     const QString DATA_DIR() {
         return m_DATA_DIR;
     }
@@ -107,14 +120,6 @@ public:
         return m_LASTOPENEDIMAGE_FILE;
     }
 
-    const QString SHAREONLINE_IMGUR_FILE() {
-        return m_SHAREONLINE_IMGUR_FILE;
-    }
-
-    const QString SHAREONLINE_IMGUR_HISTORY_DB() {
-        return m_SHAREONLINE_IMGUR_HISTORY_DB;
-    }
-
     const QString ICC_COLOR_PROFILE_DIR() {
         return m_ICC_COLOR_PROFILE_DIR;
     }
@@ -122,11 +127,7 @@ public:
     void setThumbnailCacheBaseDir(QString basedir) {
 
 #ifdef PQMPORTABLETWEAKS
-#ifdef Q_OS_WIN
-        basedir = QString("%1/photoqt-data/thumbnails/").arg(photoqt_exe_basedir);
-#else
-        basedir = QString("%1/.photoqt-data/thumbnails/").arg(photoqt_exe_basedir);
-#endif
+        basedir = QString("%1/thumbnails/").arg(qgetenv("PHOTOQT_PORTABLE_DATA_LOCATION"));
 #else
         if(basedir == "")
             basedir = QString("%1/thumbnails").arg(QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation));
@@ -150,23 +151,18 @@ private:
     PQCConfigFiles() {
 
 #ifdef PQMPORTABLETWEAKS
-
-        photoqt_exe_basedir = qgetenv("PHOTOQT_EXE_BASEDIR");
-
-    #ifdef Q_OS_WIN
-        m_CONFIG_DIR = QString("%1/photoqt-data/config/").arg(photoqt_exe_basedir);
-        m_DATA_DIR = QString("%1/photoqt-data/data/").arg(photoqt_exe_basedir);
-        m_CACHE_DIR = QString("%1/photoqt-data/cache/").arg(photoqt_exe_basedir);
-    #else
-        m_CONFIG_DIR = QString("%1/.photoqt-data/config/").arg(photoqt_exe_basedir);
-        m_DATA_DIR = QString("%1/.photoqt-data/data/").arg(photoqt_exe_basedir);
-        m_CACHE_DIR = QString("%1/.photoqt-data/cache/").arg(photoqt_exe_basedir);
-    #endif
+        const QString portablefolder = qgetenv("PHOTOQT_PORTABLE_DATA_LOCATION");
+        m_CONFIG_DIR = QString("%1/config/").arg(portablefolder);
+        m_DATA_DIR = QString("%1/data/").arg(portablefolder);
+        m_CACHE_DIR = QString("%1/cache/").arg(portablefolder);
 #else
         m_CONFIG_DIR = QStandardPaths::writableLocation(QStandardPaths::AppConfigLocation);
         m_DATA_DIR = QStandardPaths::writableLocation(QStandardPaths::AppDataLocation);
         m_CACHE_DIR = QStandardPaths::writableLocation(QStandardPaths::CacheLocation);
 #endif
+        m_EXTENSION_CONFIG_DIR = QString("%1/extensions/").arg(m_CONFIG_DIR);
+        m_EXTENSION_DATA_DIR = QString("%1/extensions/").arg(m_DATA_DIR);
+        m_EXTENSION_CACHE_DIR = QString("%1/extensions/").arg(m_CACHE_DIR);
 
         QDir dir;
         dir.mkpath(m_CONFIG_DIR);
@@ -174,11 +170,7 @@ private:
         dir.mkpath(m_CACHE_DIR);
 
 #ifdef PQMPORTABLETWEAKS
-    #ifdef Q_OS_WIN
-        m_USER_TRASH_FILES = QString("%1/photoqt-data/Trash/files").arg(photoqt_exe_basedir);
-    #else
-        m_USER_TRASH_FILES = QString("%1/.photoqt-data/Trash/files").arg(photoqt_exe_basedir);
-    #endif
+        m_USER_TRASH_FILES = QString("%1/Trash/files").arg(portablefolder);
 #else
         m_USER_TRASH_FILES = QString("%1/Trash/files").arg(QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation));
 #endif
@@ -195,11 +187,7 @@ private:
 #endif
 
 #ifdef PQMPORTABLETWEAKS
-    #ifdef Q_OS_WIN
-        m_THUMBNAIL_CACHE_DIR = QString("%1/photoqt-data/thumbnails").arg(photoqt_exe_basedir);
-    #else
-        m_THUMBNAIL_CACHE_DIR = QString("%1/.photoqt-data/thumbnails").arg(photoqt_exe_basedir);
-    #endif
+        m_THUMBNAIL_CACHE_DIR = QString("%1/thumbnails").arg(portablefolder);
 #else
         m_THUMBNAIL_CACHE_DIR = QString("%1/thumbnails").arg(QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation));
         dir.mkpath(QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation));
@@ -217,13 +205,14 @@ private:
         m_WINDOW_GEOMETRY_FILE = QString("%1/geometry").arg(CONFIG_DIR());
         m_FILEDIALOG_LAST_LOCATION = QString("%1/filedialoglastlocation").arg(CACHE_DIR());
         m_LASTOPENEDIMAGE_FILE = QString("%1/lastimageloaded").arg(CONFIG_DIR());
-        m_SHAREONLINE_IMGUR_FILE = QString("%1/imgurconfig").arg(CONFIG_DIR());
-        m_SHAREONLINE_IMGUR_HISTORY_DB = QString("%1/imgurhistory.db").arg(CONFIG_DIR());
         m_ICC_COLOR_PROFILE_DIR = QString("%1/icc").arg(CACHE_DIR());
 
     }
 
     QString m_CONFIG_DIR;
+    QString m_EXTENSION_CONFIG_DIR;
+    QString m_EXTENSION_DATA_DIR;
+    QString m_EXTENSION_CACHE_DIR;
     QString m_DATA_DIR;
     QString m_CACHE_DIR;
     QString m_USER_TRASH_FILES;
@@ -240,14 +229,8 @@ private:
     QString m_WINDOW_GEOMETRY_FILE;
     QString m_FILEDIALOG_LAST_LOCATION;
     QString m_LASTOPENEDIMAGE_FILE;
-    QString m_SHAREONLINE_IMGUR_FILE;
-    QString m_SHAREONLINE_IMGUR_HISTORY_DB;
     QString m_ICC_COLOR_PROFILE_DIR;
 
-#ifdef PQMPORTABLETWEAKS
-    QString photoqt_exe_basedir;
-#endif
-
 };
 
 #endif // PQCCONFIGFILES_H
