@@ -24,6 +24,7 @@
 #include <QFileInfo>
 #include <QDir>
 #include <thread>
+#include <iostream>
 #include <qlogging.h>   // needed in this form to compile with Qt 6.2
 
 #include <QQmlApplicationEngine>
@@ -34,10 +35,9 @@
 
 #include <pqc_commandlineparser.h>
 #include <pqc_singleinstance.h>
-#include <pqc_notify.h>
-#include <pqc_settings.h>
+#include <pqc_notify_cpp.h>
 #include <pqc_configfiles.h>
-#include <pqc_filefoldermodel.h>
+#include <pqc_filefoldermodelCPP.h>
 
 PQCSingleInstance::PQCSingleInstance(int &argc, char *argv[]) : QApplication(argc, argv) {
 
@@ -53,13 +53,16 @@ PQCSingleInstance::PQCSingleInstance(int &argc, char *argv[]) : QApplication(arg
     socket = nullptr;
     server = nullptr;
 
+    forceModernInterface = false;
+    forceIntegratedInterface = false;
+
+    m_forceShowWizard = false;
+    m_forceSkipWizard = false;
+
     if(result & PQCCommandLineFile) {
         for(const auto &f : std::as_const(parser.filenames)) {
-            QString ff = f;
-            if(!QFileInfo(ff).isAbsolute())
-                ff = QDir::currentPath() + "/" + ff;
             msg << Actions::File;
-            receivedFile = QFileInfo(ff).canonicalFilePath();
+            receivedFile = QFileInfo(f).canonicalFilePath();
         }
     }
 
@@ -103,12 +106,28 @@ PQCSingleInstance::PQCSingleInstance(int &argc, char *argv[]) : QApplication(arg
     if(result & PQCCommandLineNoDebug)
         msg << Actions::NoDebug;
 
+    if(result & PQCCommandLineModernInterface)
+        forceModernInterface = true;
+
+    if(result & PQCCommandLineIntegratedInterface)
+        forceIntegratedInterface = true;
+
+    if(result & PQCCommandLineShowStartupWizard)
+        m_forceShowWizard = true;
+
+    if(result & PQCCommandLineDontShowStartupWizard)
+        m_forceSkipWizard = true;
+
     if(result & PQCCommandLineSettingUpdate) {
         receivedSetting[0] = parser.settingUpdate[0];
         receivedSetting[1] = parser.settingUpdate[1];
         msg << Actions::Setting;
     }
 
+    if(result & PQCCommandLineInstallExtension) {
+        installExtensionFileName = parser.extensionFileName;
+    }
+
     // validation requested
     checkConfig = false;
     if(result & PQCCommandLineCheckConfig) {
@@ -156,13 +175,19 @@ PQCSingleInstance::PQCSingleInstance(int &argc, char *argv[]) : QApplication(arg
 
     // we need to figure out if multiple instances are allowed here WITHOUT using the PQCSettings class
     if(QFile::exists(PQCConfigFiles::get().USERSETTINGS_DB())) {
+
+        // This database connection happens before the general setup in PQCStartupManager
         QSqlDatabase dbtmp;
-        if(QSqlDatabase::isDriverAvailable("QSQLITE3"))
-            dbtmp = QSqlDatabase::addDatabase("QSQLITE3", "settingsmultiple");
-        else if(QSqlDatabase::isDriverAvailable("QSQLITE"))
-            dbtmp = QSqlDatabase::addDatabase("QSQLITE", "settingsmultiple");
-        dbtmp.setConnectOptions("QSQLITE_OPEN_READONLY");
-        dbtmp.setDatabaseName(PQCConfigFiles::get().USERSETTINGS_DB());
+        if(QSqlDatabase::contains("settings"))
+            dbtmp = QSqlDatabase::database("settings");
+        else {
+            if(QSqlDatabase::isDriverAvailable("QSQLITE3"))
+                dbtmp = QSqlDatabase::addDatabase("QSQLITE3", "settings");
+            else if(QSqlDatabase::isDriverAvailable("QSQLITE"))
+                dbtmp = QSqlDatabase::addDatabase("QSQLITE", "settings");
+            dbtmp.setDatabaseName(PQCConfigFiles::get().USERSETTINGS_DB());
+        }
+
         if(!dbtmp.open()) {
             qWarning() << "Unable to check how to handle multiple instances:" << dbtmp.lastError().text();
             qWarning() << "Assuming only a single instance is to be used";
@@ -210,20 +235,32 @@ PQCSingleInstance::PQCSingleInstance(int &argc, char *argv[]) : QApplication(arg
         if(msg.size() == 0)
             msg << Actions::Show;
 
-        QStringList _strings;
-        _strings.reserve(msg.size());
+        QList<QByteArray> writeMessage;
+
+        if(qApp->platformName() == "wayland") {
+
+            writeMessage.reserve(msg.size()+1);
+
+            QString token = qgetenv("XDG_ACTIVATION_TOKEN");
+            if(!token.isEmpty())
+                writeMessage.append(QStringLiteral("_T_O_K_E_N_%1\n").arg(token).toUtf8());
+
+        } else
+            writeMessage.reserve(msg.size());
+
         for(const Actions &i : std::as_const(msg)) {
-            _strings.append(QString::number(static_cast<int>(i)));
+            writeMessage.append(QString::number(static_cast<int>(i)).toUtf8());
         }
 
         // Send composed message string
         if(receivedFile != "")
-            socket->write(QStringLiteral("_F_I_L_E_%1\n").arg(receivedFile).toUtf8());
+            writeMessage.append(QStringLiteral("_F_I_L_E_%1\n").arg(receivedFile).toUtf8());
         if(receivedShortcut != "")
-            socket->write(QStringLiteral("_S_H_O_R_T_C_U_T_%1\n").arg(receivedShortcut).toUtf8());
+            writeMessage.append(QStringLiteral("_S_H_O_R_T_C_U_T_%1\n").arg(receivedShortcut).toUtf8());
         if(receivedSetting[0] != "")
-            socket->write(QStringLiteral("_S_E_T_T_I_N_G_%1:%2\n").arg(receivedSetting[0], receivedSetting[1]).toUtf8());
-        socket->write(QStringLiteral("%1\n").arg(_strings.join('/')).toUtf8());
+            writeMessage.append(QStringLiteral("_S_E_T_T_I_N_G_%1:%2\n").arg(receivedSetting[0], receivedSetting[1]).toUtf8());
+
+        socket->write(writeMessage.join('\n'));
         socket->flush();
 
         // Inform user
@@ -246,7 +283,10 @@ PQCSingleInstance::PQCSingleInstance(int &argc, char *argv[]) : QApplication(arg
         m_receivedSetting[0] = receivedSetting[0];
         m_receivedSetting[1] = receivedSetting[1];
         m_receivedShortcut = receivedShortcut;
+
         handleMessage(msg);
+        m_startupMessageForProcessing = msg;
+        connect(&PQCNotifyCPP::get(), &PQCNotifyCPP::reprocessStartupMessage, this, [=]() { handleMessage(m_startupMessageForProcessing, false); });
 
     }
 
@@ -255,35 +295,44 @@ PQCSingleInstance::PQCSingleInstance(int &argc, char *argv[]) : QApplication(arg
 void PQCSingleInstance::newConnection() {
     QLocalSocket *socket = server->nextPendingConnection();
     if(socket->waitForReadyRead(2000)) {
-        QByteArray rep = socket->readAll().split('\n')[0];
-        if(rep.startsWith("_F_I_L_E_")) {
-            m_receivedFile = rep.last(rep.length()-9);
-            handleMessage({Actions::File});
-        } else if(rep.startsWith("_S_H_O_R_T_C_U_T_")) {
-            m_receivedShortcut = rep.last(rep.length()-17);
-            handleMessage({Actions::Shortcut});
-        } else if(rep.startsWith("_S_E_T_T_I_N_G_")) {
-            const QList<QByteArray> tmp = rep.last(rep.length()-15).split(':');
-            m_receivedSetting[0] = tmp[0];
-            m_receivedSetting[1] = tmp[1];
-            handleMessage({Actions::Setting});
-        } else {
-            QList<Actions> _ints;
-            const QList<QByteArray> _reps = rep.split('/');
-            _ints.reserve(_reps.size());
-            for(const QByteArray &r : _reps) {
-                _ints << static_cast<Actions>(r.toInt());
+        const QList<QByteArray> reply = socket->readAll().split('\n');
+        QList<Actions> handleAll;
+        for(const QByteArray &rep : reply) {
+            if(rep.startsWith("_T_O_K_E_N_")) {
+                qputenv("XDG_ACTIVATION_TOKEN", rep.last(rep.length()-11));
+            } else if(rep.startsWith("_F_I_L_E_")) {
+                m_receivedFile = rep.last(rep.length()-9);
+                handleAll.append(Actions::File);
+            } else if(rep.startsWith("_S_H_O_R_T_C_U_T_")) {
+                m_receivedShortcut = rep.last(rep.length()-17);
+                handleAll.append(Actions::Shortcut);
+            } else if(rep.startsWith("_S_E_T_T_I_N_G_")) {
+                const QList<QByteArray> tmp = rep.last(rep.length()-15).split(':');
+                m_receivedSetting[0] = tmp[0];
+                m_receivedSetting[1] = tmp[1];
+                handleAll.append(Actions::Setting);
+            } else {
+                const QList<QByteArray> _reps = rep.split('/');
+                for(const QByteArray &r : _reps) {
+                    handleAll.append(static_cast<Actions>(r.toInt()));
+                }
             }
-            handleMessage(_ints);
         }
+        handleMessage(handleAll);
     }
     socket->close();
     delete socket;
 }
 
-void PQCSingleInstance::handleMessage(const QList<Actions> msg) {
+void PQCSingleInstance::handleMessage(const QList<Actions> msg, bool includeFileProcessing) {
 
     qDebug() << "args: msg";
+    qDebug() << "args: includeFileProcessing =" << includeFileProcessing;
+
+    // NOTE
+    // file processing is handled directly during startup for faster displaying of files
+    // all others might need to be re-processed once the ui is ready
+    // that's why this call is also triggered from qml at the end of startup
 
     QStringList allfiles;
     QStringList allfolders;
@@ -296,74 +345,95 @@ void PQCSingleInstance::handleMessage(const QList<Actions> msg) {
 
         case Actions::File:
 
-            // sort by files and folders
-            // that way we can make sure to always load the first specified file as initial image
-            if(!info.exists())
-                continue;
-            if(info.isFile())
-                allfiles.append(m_receivedFile);
-            else if(info.isDir())
-                allfolders.append(m_receivedFile);
+            if(includeFileProcessing) {
+
+
+                // sort by files and folders
+                // that way we can make sure to always load the first specified file as initial image
+                if(!info.exists())
+                    continue;
+                if(info.isFile()) {
+                    qDebug() << "received file:" << m_receivedFile;
+                    allfiles.append(m_receivedFile);
+                } else if(info.isDir()) {
+                    qDebug() << "received folder:" << m_receivedFile;
+                    allfolders.append(m_receivedFile);
+                }
+
+            }
+
             break;
 
         case Actions::Open:
 
-            Q_EMIT PQCNotify::get().cmdOpen();
+            qDebug() << "request action: open";
+            Q_EMIT PQCNotifyCPP::get().cmdOpen();
             break;
 
         case Actions::Show:
 
-            Q_EMIT PQCNotify::get().cmdShow();
+            qDebug() << "request action: show";
+            Q_EMIT PQCNotifyCPP::get().cmdShow();
             break;
 
         case Actions::Hide:
 
-            Q_EMIT PQCNotify::get().cmdHide();
+            qDebug() << "request action: hide";
+            Q_EMIT PQCNotifyCPP::get().cmdHide();
             break;
 
         case Actions::Quit:
 
-            Q_EMIT PQCNotify::get().cmdQuit();
+            qDebug() << "request action: quit";
+            Q_EMIT PQCNotifyCPP::get().cmdQuit();
             break;
 
         case Actions::Toggle:
 
-            Q_EMIT PQCNotify::get().cmdToggle();
+            qDebug() << "request action: toggle";
+            Q_EMIT PQCNotifyCPP::get().cmdToggle();
             break;
 
         case Actions::StartInTray:
 
-            PQCNotify::get().setStartInTray(true);
+            qDebug() << "request action: strat in tray";
+            PQCNotifyCPP::get().setStartInTray(true);
             break;
 
         case Actions::Tray:
 
-            Q_EMIT PQCNotify::get().cmdTray(true);
+            qDebug() << "request action: tray";
+            Q_EMIT PQCNotifyCPP::get().cmdTray(true);
             break;
 
         case Actions::NoTray:
 
-            Q_EMIT PQCNotify::get().cmdTray(false);
+            qDebug() << "request action: no tray";
+            Q_EMIT PQCNotifyCPP::get().cmdTray(false);
             break;
 
         case Actions::Shortcut:
 
-            Q_EMIT PQCNotify::get().cmdShortcutSequence(m_receivedShortcut);
+            qDebug() << "request action: shortcut:" << m_receivedShortcut;
+            Q_EMIT PQCNotifyCPP::get().cmdShortcutSequence(m_receivedShortcut);
             break;
 
         case Actions::Debug:
 
-            PQCNotify::get().setDebug(true);
+            qDebug() << "request action: debug";
+            PQCNotifyCPP::get().setDebug(true);
             break;
 
         case Actions::NoDebug:
 
-            PQCNotify::get().setDebug(false);
+            qDebug() << "request action: no debug";
+            PQCNotifyCPP::get().setDebug(false);
             break;
 
         case Actions::Setting:
 
-            PQCNotify::get().setSettingUpdate({m_receivedSetting[0], m_receivedSetting[1]});
+            qDebug() << "request action: settings update:" << m_receivedSetting;
+            PQCNotifyCPP::get().setSettingUpdate({m_receivedSetting[0], m_receivedSetting[1]});
             break;
 
         default:
@@ -373,14 +443,17 @@ void PQCSingleInstance::handleMessage(const QList<Actions> msg) {
 
     }
 
-    // if we have files and/or folders that were passed on
-    if(allfiles.length() > 0 || allfolders.length() > 0) {
-        allfiles.append(allfolders);
-        if(allfiles.length() > 1)
-            PQCFileFolderModel::get().setExtraFoldersToLoad(allfiles.mid(1));
-        else
-            PQCFileFolderModel::get().setExtraFoldersToLoad({});
-        PQCNotify::get().setFilePath(allfiles[0]);
+    if(includeFileProcessing) {
+        // if we have files and/or folders that were passed on
+        if(allfiles.length() > 0 || allfolders.length() > 0) {
+            allfiles.append(allfolders);
+            if(allfiles.length() > 1)
+                Q_EMIT PQCFileFolderModelCPP::get().setExtraFoldersToLoad(allfiles.mid(1));
+            else
+                Q_EMIT PQCFileFolderModelCPP::get().setExtraFoldersToLoad({});
+            qDebug() << "request action: set file:" << allfiles[0];
+            PQCNotifyCPP::get().setFilePath(allfiles[0]);
+        }
     }
 
 }
@@ -388,19 +461,32 @@ void PQCSingleInstance::handleMessage(const QList<Actions> msg) {
 bool PQCSingleInstance::notify(QObject *obj, QEvent *e) {
 
     const QString cn = obj->metaObject()->className();
+
     if(cn == "QQuickRootItem") {
         if(e->type() == QEvent::KeyPress) {
             QKeyEvent *ev = reinterpret_cast<QKeyEvent*>(e);
-            Q_EMIT PQCNotify::get().keyPress(ev->key(), ev->modifiers());
+            Q_EMIT PQCNotifyCPP::get().keyPress(ev->key(), ev->modifiers());
         } else if(e->type() == QEvent::KeyRelease) {
             QKeyEvent *ev = reinterpret_cast<QKeyEvent*>(e);
-            Q_EMIT PQCNotify::get().keyRelease(ev->key(), ev->modifiers());
+            Q_EMIT PQCNotifyCPP::get().keyRelease(ev->key(), ev->modifiers());
+        }
+    } else if(cn == "QQuickPopupItem") {
+        // we process modifiers only for popups (like menus)
+        if(e->type() == QEvent::KeyPress) {
+            QKeyEvent *ev = reinterpret_cast<QKeyEvent*>(e);
+            if((ev->key() == Qt::Key_Shift && ev->modifiers() == Qt::ShiftModifier) ||
+               (ev->key() == Qt::Key_Alt && ev->modifiers() == Qt::AltModifier) ||
+               (ev->key() == Qt::Key_Control && ev->modifiers() == Qt::ControlModifier))
+                Q_EMIT PQCNotifyCPP::get().keyPress(ev->key(), ev->modifiers());
+        } else if(e->type() == QEvent::KeyRelease) {
+            QKeyEvent *ev = reinterpret_cast<QKeyEvent*>(e);
+            Q_EMIT PQCNotifyCPP::get().keyRelease(ev->key(), ev->modifiers());
         }
     } else if(cn.startsWith("PQMainWindow")) {
         if(e->type() == QEvent::Leave) {
-            Q_EMIT PQCNotify::get().mouseWindowExit();
+            Q_EMIT PQCNotifyCPP::get().mouseWindowExit();
         } else if(e->type() == QEvent::Enter)
-            Q_EMIT PQCNotify::get().mouseWindowEnter();
+            Q_EMIT PQCNotifyCPP::get().mouseWindowEnter();
     }
 
     return QApplication::notify(obj, e);
