@@ -27,17 +27,14 @@
 #include <QImageReader>
 #include <QQmlContext>
 #include <QMessageBox>
+#include <QDirIterator>
 #include <pqc_configfiles.h>
-#include <pqc_settings.h>
-#include <pqc_shortcuts.h>
-#include <pqc_validate.h>
-#include <pqc_imageformats.h>
 #include <scripts/pqc_scriptsconfig.h>
-#include <scripts/pqc_scriptscontextmenu.h>
-#include <scripts/pqc_scriptsshareimgur.h>
-#include <pqc_startup.h>
-#include <pqc_location.h>
-#include <pqc_notify.h>
+#include <scripts/pqc_scriptslocalization.h>
+#ifndef PQMTESTING
+#include <pqc_startuphandler.h>
+#endif
+#include <pqc_notify_cpp.h>
 
 #ifdef WIN32
 #include <WinSock2.h>
@@ -84,14 +81,9 @@
 #include <lcms2.h>
 #endif
 
-PQCScriptsConfig::PQCScriptsConfig() {
-    trans = new QTranslator;
-    currentTranslation = "en";
-}
+PQCScriptsConfig::PQCScriptsConfig() {}
 
-PQCScriptsConfig::~PQCScriptsConfig() {
-    delete trans;
-}
+PQCScriptsConfig::~PQCScriptsConfig() {}
 
 bool PQCScriptsConfig::amIOnWindows() {
 #ifdef Q_OS_WIN
@@ -119,6 +111,10 @@ QString PQCScriptsConfig::getConfigInfo(bool formatHTML) {
 
     txt += QString(" - Compiled with %1Qt %2%3, running with %4Qt %5%6%7").arg(bold1, QT_VERSION_STR, bold2, bold1, qVersion(), bold2, nl);
 
+#ifdef PQMPORTABLETWEAKS
+    txt += QString(" - Portable version%1").arg(nl);
+#endif
+
 #ifdef PQMEXIV2
     txt += QString(" - %1Exiv2%2: %3%4").arg(bold1, bold2, Exiv2::version(), nl);
 #endif
@@ -163,6 +159,10 @@ QString PQCScriptsConfig::getConfigInfo(bool formatHTML) {
     txt += QString(" - %1DevIL%2: %3%4").arg(bold1, bold2).arg(IL_VERSION).arg(nl);
 #endif
 
+#ifdef PQMLIBSAI
+    txt += QString(" - %1LibSai%2%3").arg(bold1, bold2).arg(nl);
+#endif
+
 #ifdef PQMLOCATION
     txt += QString(" - %1Location%2%3").arg(bold1, bold2, nl);
 #endif
@@ -203,6 +203,15 @@ QString PQCScriptsConfig::getConfigInfo(bool formatHTML) {
 
 bool PQCScriptsConfig::exportConfigTo(QString path) {
 
+    /*********************************************************/
+    // NOTE: BEFORE CALLING you HAVE TO ENSURE:
+    // -> settings database is closed
+    // -> shortcuts database is closed
+    //
+    // If called by command line, they are never opened.
+    // If called through interface, they must be closed (and subsequently reopened) manually!
+    /*********************************************************/
+
     qDebug() << "args: path =" << path;
 
 #ifdef PQMLIBARCHIVE
@@ -272,7 +281,7 @@ bool PQCScriptsConfig::exportConfigTo(QString path) {
                 archive_entry_free(entry);
 
             } else
-                qWarning() <<  QString("PQHandlingExternal::exportConfig(): ERROR: Unable to read config file '%1'... Skipping!").arg(iter.value());
+                qWarning() <<  QString("ERROR: Unable to read config file '%1'... Skipping!").arg(iter.value());
         }
         ++iter;
     }
@@ -290,14 +299,20 @@ bool PQCScriptsConfig::exportConfigTo(QString path) {
 
 bool PQCScriptsConfig::importConfigFrom(QString path) {
 
+    /*********************************************************/
+    // NOTE: BEFORE CALLING you HAVE TO ENSURE:
+    // -> settings database is closed
+    // -> shortcuts database is closed
+    // -> image formats database is closed
+    //
+    // If called by command line, they are never opened.
+    // If called through interface, they must be closed manually!
+    /*********************************************************/
+
     qDebug() << "args: path =" << path;
 
 #ifdef PQMLIBARCHIVE
 
-    PQCSettings::get().closeDatabase();
-    PQCShortcuts::get().closeDatabase();
-    PQCImageFormats::get().closeDatabase();
-
     // Obtain a filename from the user or used passed on filename
     QString archiveFile;
     if(path == "") {
@@ -537,108 +552,27 @@ QString PQCScriptsConfig::getVersion() {
     return PQMVERSION;
 }
 
-QStringList PQCScriptsConfig::getAvailableTranslations() {
-
-    qDebug() << "";
-
-    QStringList ret;
-
-    QStringList tmp;
-
-    // the non-translated language is English
-    tmp << "en";
-
-    QDirIterator it(":/lang");
-    while (it.hasNext()) {
-        QString file = it.next();
-        if(file.endsWith(".qm")) {
-            file = file.remove(0, 15);
-            file = file.remove(file.length()-3, file.length());
-            if(!ret.contains(file))
-                tmp.push_back(file);
-        }
-    }
-
-    tmp.sort();
-    ret.append(tmp);
-
-    return ret;
-
-}
-
-void PQCScriptsConfig::updateTranslation() {
-
-    qDebug() << "";
-
-    QString code = PQCSettings::get()["interfaceLanguage"].toString();
-    if(code == currentTranslation)
-        return;
-
-    static QTranslator trans;
-    qApp->removeTranslator(&trans);
-
-    const QStringList allcodes = code.split("/");
-
-    // we use this to detect whether a translation was found for the above language code
-    currentTranslation = "";
-    for(const QString &c : allcodes) {
-
-        if(QFile(":/lang/photoqt_" + c + ".qm").exists()) {
-
-            if(trans.load(":/lang/photoqt_" + c)) {
-                currentTranslation = c;
-                qApp->installTranslator(&trans);
-            } else
-                qWarning() << "Unable to install translator for language code" << c;
-
-        } else if(c.contains("_")) {
-
-            const QString cc = c.split("_").at(0);
-
-            if(QFile(":/lang/photoqt_" + cc + ".qm").exists()) {
-
-                if(trans.load(":/lang/photoqt_" + cc)) {
-                    currentTranslation = cc;
-                    qApp->installTranslator(&trans);
-                } else
-                    qWarning() << "Unable to install translator for language code" << cc;
-
-            }
-
-        } else {
-
-            const QString cc = QString("%1_%2").arg(c, c.toUpper());
-
-            if(QFile(":/lang/photoqt_" + cc + ".qm").exists()) {
-
-                if(trans.load(":/lang/photoqt_" + cc)) {
-                    currentTranslation = cc;
-                    qApp->installTranslator(&trans);
-                } else
-                    qWarning() << "Unable to install translator for language code" << c;
-
-            }
-        }
-
-    }
-
-    // no translation found -> store selected code
-    if(currentTranslation == "")
-        currentTranslation = code;
-
-    QQmlEngine::contextForObject(this)->engine()->retranslate();
-
-}
-
 bool PQCScriptsConfig::isBetaVersion() {
     return QString(PQMVERSION).contains("-beta");
 }
 
+bool PQCScriptsConfig::isDebugBuild() {
+#ifdef NDEBUG
+    return false;
+#else
+    return true;
+#endif
+}
+
 void PQCScriptsConfig::inform(QString title, QString txt) {
 
-    QMessageBox::information(0,
-                             title,
-                             txt);
+    QMessageBox msg;
+    msg.setIcon(QMessageBox::Information);
+    msg.setWindowFlag(Qt::WindowStaysOnTopHint);
+    msg.setWindowTitle(title);
+    msg.setText(txt);
+    msg.setStandardButtons(QMessageBox::Ok);
+    msg.exec();
 
 }
 
@@ -670,17 +604,6 @@ bool PQCScriptsConfig::isLCMS2SupportEnabled() {
     return false;
 }
 
-void PQCScriptsConfig::setDefaultSettingValueFor(QString key) {
-    PQCSettings::get().setDefaultFor(key);
-}
-
-QVariant PQCScriptsConfig::getDefaultSettingValueFor(QString key) {
-    const QVariantList val = PQCSettings::get().getDefaultFor(key);
-    if(val[1].toString() == "list")
-        return val[0].toString().split(":://::");
-    return val[0];
-}
-
 bool PQCScriptsConfig::isICUSupportEnabled() {
 #ifdef PQMWITHOUTICU
     return false;
@@ -688,53 +611,49 @@ bool PQCScriptsConfig::isICUSupportEnabled() {
     return true;
 }
 
-void PQCScriptsConfig::resetToDefaultsWithConfirmation(bool skipConfirmation) {
+void PQCScriptsConfig::callStartupSetupFresh() {
 
-    PQCNotify::get().setIgnoreAllKeys(true);
-
-    int ret = QMessageBox::Yes;
-
-    if(!skipConfirmation) {
-        QMessageBox msg;
-        msg.setWindowTitle(QApplication::translate("configuration", "Reset PhotoQt to its default state."));
-        msg.setText(QApplication::translate("configuration", "Do you want to reset PhotoQt to its default state? If you encounter any issues with your configuration, you should be able to fix it this way."));
-        msg.setInformativeText("<b>" + QApplication::translate("configuration", "Warning: This step cannot be undone!") + "</b>");
-        msg.setStandardButtons(QMessageBox::Yes | QMessageBox::Cancel);
-        msg.setDefaultButton(QMessageBox::Yes);
-        ret = msg.exec();
-    }
-
-    if(ret == QMessageBox::Yes) {
-
-        qDebug() << "RESETTING PHOTOQT TO ITS DEFAULTS";
-
-        PQCSettings::get().closeDatabase();
-        PQCShortcuts::get().closeDatabase();
-        PQCImageFormats::get().closeDatabase();
-        PQCLocation::get().closeDatabase();
-        PQCScriptsContextMenu::get().closeDatabase();
-        PQCScriptsShareImgur::get().closeDatabase();
-
-        PQCStartup startup;
-        startup.setupFresh();
-
-        // further setup for settings
-        PQCSettings::get().reopenDatabase();
-        PQCSettings::get().setupFresh();
-
-        // further setup for shortcuts
-        PQCShortcuts::get().reopenDatabase();
-        PQCShortcuts::get().setupFresh();
-
-        if(!skipConfirmation) {
-            QMessageBox::information(0, QApplication::translate("configuration", "Restart PhotoQt"),
-                                     QApplication::translate("configuration", "PhotoQt has been reset to its defaults and will need to be restarted."));
-            qApp->quit();
-        }
-
-
-    }
-
-    PQCNotify::get().setIgnoreAllKeys(false);
+#ifndef PQMTESTING
+    PQCStartupHandler startup(false, false);
+    startup.setupFresh();
+#endif
 
 }
+
+bool PQCScriptsConfig::askForConfirmation(QString title, QString text, QString informativeText) {
+    QMessageBox msg;
+    msg.setIcon(QMessageBox::Question);
+    msg.setWindowFlag(Qt::WindowStaysOnTopHint);
+    msg.setWindowTitle(title);
+    msg.setText(text);
+    msg.setInformativeText(informativeText);
+    msg.setStandardButtons(QMessageBox::Yes | QMessageBox::Cancel);
+    msg.setDefaultButton(QMessageBox::Yes);
+    return (msg.exec() == QMessageBox::Yes);
+}
+
+bool PQCScriptsConfig::setInterfaceForNextStartup(QString variant) {
+    QFile file(PQCConfigFiles::get().CACHE_DIR() + "/nextstartupvariant");
+    if(!file.open(QIODevice::WriteOnly|QIODevice::Truncate)) {
+        qWarning() << "Unable to toggle interface variant";
+        return false;
+    }
+    QTextStream out(&file);
+    out << variant;
+    file.close();
+    return true;
+}
+
+QString PQCScriptsConfig::getInterfaceForNextStartup() {
+    QFile file(PQCConfigFiles::get().CACHE_DIR() + "/nextstartupvariant");
+    if(!file.exists()) return "";
+    if(!file.open(QIODevice::ReadOnly)) {
+        qWarning() << "Unable to read toggled interface variant";
+        return "";
+    }
+    QTextStream in(&file);
+    QString ret = in.readAll().trimmed();
+    if(ret == "modern" || ret == "integrated")
+        return ret;
+    return "";
+}
