@@ -29,18 +29,22 @@
 #include <QProcess>
 #include <QStringConverter>
 #include <QImageReader>
-#include <QtConcurrent>
+#include <QtConcurrent/QtConcurrentRun>
 #include <QMediaPlayer>
 #include <QColorSpace>
 #include <QFileDialog>
 #include <QScreen>
+#include <QMimeDatabase>
+#include <QCollator>
+#include <QCryptographicHash>
+
 #include <scripts/pqc_scriptsimages.h>
 #include <scripts/pqc_scriptsfilespaths.h>
 #include <pqc_settingscpp.h>
 #include <pqc_imageformats.h>
 #include <pqc_loadimage.h>
 #include <pqc_configfiles.h>
-#include <pqc_notify.h>
+#include <pqc_notify_cpp.h>
 
 #ifdef PQMWAYLANDSPECIFIC
 #include <pqc_wayland.h>
@@ -76,6 +80,11 @@
 #include <lcms2.h>
 #endif
 
+PQCScriptsImages &PQCScriptsImages::get() {
+    static PQCScriptsImages instance;
+    return instance;
+}
+
 PQCScriptsImages::PQCScriptsImages() {
     // importedICCLastMod = 0;
     // colorlastlocation = new QFile(QString("%1/%2").arg(PQCConfigFiles::get().CACHE_DIR(), "colorlastlocation"));
@@ -92,9 +101,7 @@ PQCScriptsImages::PQCScriptsImages() {
 
 }
 
-PQCScriptsImages::~PQCScriptsImages() {
-    // delete colorlastlocation;
-}
+PQCScriptsImages::~PQCScriptsImages() {}
 
 QSize PQCScriptsImages::getCurrentImageResolution(QString filename) {
 
@@ -172,7 +179,7 @@ QString PQCScriptsImages::loadImageAndConvertToBase64(QString filename) {
 
 }
 
-QStringList PQCScriptsImages::listArchiveContent(QString path, bool insideFilenameOnly) {
+void PQCScriptsImages::listArchiveContent(QString path, bool insideFilenameOnly) {
 
     qDebug() << "args: path =" << path;
     qDebug() << "args: insideFilenameOnly =" << insideFilenameOnly;
@@ -183,11 +190,26 @@ QStringList PQCScriptsImages::listArchiveContent(QString path, bool insideFilena
     const QFileInfo info(path);
     QString cacheKey = QString("%1::%2::%3::%4").arg(info.lastModified().toMSecsSinceEpoch()).arg(path, PQCSettingsCPP::get().getImageviewSortImagesAscending()).arg(insideFilenameOnly);
 
-    if(archiveContentCache.contains(cacheKey))
-        return archiveContentCache[cacheKey];
+    if(archiveContentCache.contains(cacheKey)) {
+        Q_EMIT haveArchiveContentFor(path, archiveContentCache[cacheKey]);
+        return;
+    }
+
+    QFuture<void> f = QtConcurrent::run([=]() {
+        Q_EMIT haveArchiveContentFor(path, PQCScriptsImages::listArchiveContentWithoutThread(path, cacheKey, insideFilenameOnly));
+    });
+
+}
+
+QStringList PQCScriptsImages::listArchiveContentWithoutThread(QString path, QString cacheKey, bool insideFilenameOnly) {
 
     QStringList ret;
 
+    const QFileInfo info(path);
+
+    if(cacheKey == "") {
+        cacheKey = QString("%1::%2::%3::%4").arg(info.lastModified().toMSecsSinceEpoch()).arg(path, PQCSettingsCPP::get().getImageviewSortImagesAscending()).arg(insideFilenameOnly);
+    }
 
 #ifndef Q_OS_WIN
 
@@ -339,108 +361,6 @@ QString PQCScriptsImages::convertSecondsToPosition(int t) {
 
 }
 
-void PQCScriptsImages::loadHistogramData(QString filepath, int index) {
-
-    QFuture<void> f = QtConcurrent::run([=]() {
-        _loadHistogramData(filepath, index);
-    });
-
-}
-
-void PQCScriptsImages::_loadHistogramData(QString filepath, int index) {
-
-    qDebug() << "args: filepath =" << filepath;
-
-    QFileInfo info(filepath);
-    QString key = QString("%1%2").arg(filepath).arg(info.lastModified().toMSecsSinceEpoch());
-    if(histogramCache.contains(key)) {
-        Q_EMIT histogramDataLoaded(histogramCache[key], index);
-        return;
-    }
-
-    QVariantList ret;
-
-    if(filepath == "" || !info.exists()) {
-        Q_EMIT histogramDataLoadedFailed(index);
-        return;
-    }
-
-    // first we need to retrieve the current image
-    QImage img;
-    QSize size;
-    PQCLoadImage::get().load(filepath, QSize(), size, img);
-
-    if(img.size().isNull() || img.size().isEmpty()) {
-        Q_EMIT histogramDataLoadedFailed(index);
-        return;
-    }
-
-    if(img.format() != QImage::Format_RGB32)
-        img.convertTo(QImage::Format_RGB32);
-
-    // we first count using integers for faster adding up
-    QList<int> red(256);
-    QList<int> green(256);
-    QList<int> blue(256);
-
-    // Loop over all rows of the image
-    for(int i = 0; i < img.height(); ++i) {
-
-        // Get the pixel data of row i of the image
-        QRgb *rowData = (QRgb*)img.scanLine(i);
-
-        // Loop over all columns
-        for(int j = 0; j < img.width(); ++j) {
-
-            // Get pixel data of pixel at column j in row i
-            QRgb pixelData = rowData[j];
-
-            // store color data
-            ++red[qRed(pixelData)];
-            ++green[qGreen(pixelData)];
-            ++blue[qBlue(pixelData)];
-
-        }
-
-    }
-
-    // we compute the grey values once we red all rgb pixels
-    // this is much faster than calculate the grey values for each pixel
-    QList<int> grey(256);
-    for(int i = 0; i < 256; ++i)
-        grey[i] = red[i]*0.34375 + green[i]*0.5 + blue[i]*0.15625;
-
-    // find the max values for normalization
-    double max_red = *std::max_element(red.begin(), red.end());
-    double max_green = *std::max_element(green.begin(), green.end());
-    double max_blue = *std::max_element(blue.begin(), blue.end());
-    double max_grey = *std::max_element(grey.begin(), grey.end());
-    double max_rgb = qMax(max_red, qMax(max_green, max_blue));
-
-    // the return lists, normalized
-    QList<float> ret_red(256);
-    QList<float> ret_green(256);
-    QList<float> ret_blue(256);
-    QList<float> ret_gray(256);
-
-    // normalize values
-    std::transform(red.begin(), red.end(), ret_red.begin(), [=](float val) { return val/max_rgb; });
-    std::transform(green.begin(), green.end(), ret_green.begin(), [=](float val) { return val/max_rgb; });
-    std::transform(blue.begin(), blue.end(), ret_blue.begin(), [=](float val) { return val/max_rgb; });
-    std::transform(grey.begin(), grey.end(), ret_gray.begin(), [=](float val) { return val/max_grey; });
-
-    // store values
-    ret << QVariant::fromValue(ret_red);
-    ret << QVariant::fromValue(ret_green);
-    ret << QVariant::fromValue(ret_blue);
-    ret << QVariant::fromValue(ret_gray);
-
-    histogramCache.insert(key, ret);
-
-    Q_EMIT histogramDataLoaded(ret, index);
-
-}
-
 bool PQCScriptsImages::isMpvVideo(QString path) {
 
     qDebug() << "args: path =" << path;
@@ -743,7 +663,11 @@ QString PQCScriptsImages::extractMotionPhoto(QString path) {
 
                 // write video to temporary file
                 QFile outfile(videofilename);
-                outfile.open(QIODevice::WriteOnly);
+                if(!outfile.open(QIODevice::WriteOnly)) {
+                    delete[] data;
+                    qWarning() << "ERROR extracting motion photo.";
+                    return "";
+                }
                 QDataStream out(&outfile);
                 out.writeRawData(videodata, info.size()-i);
                 outfile.close();
@@ -849,8 +773,42 @@ QVariantList PQCScriptsImages::getZXingData(QString path) {
     QImage img;
     PQCLoadImage::get().load(path, QSize(-1,-1), origSize, img);
 
-    // convert to gray scale
-    img.convertTo(QImage::Format_Grayscale8);
+    ZXing::ImageFormat frmt;
+    switch (img.format()) {
+        case QImage::Format_ARGB32:
+        case QImage::Format_RGB32:
+#if ZXING_VERSION_MAJOR <=2 && ZXING_VERSION_MINOR <= 2
+    #if Q_BYTE_ORDER == Q_LITTLE_ENDIAN
+            frmt = ZXing::ImageFormat::BGRX;
+    #else
+            frmt = ZXing::ImageFormat::XRGB;
+    #endif
+#else
+    #if Q_BYTE_ORDER == Q_LITTLE_ENDIAN
+            frmt = ZXing::ImageFormat::BGRA;
+    #else
+            frmt = ZXing::ImageFormat::ARGB;
+    #endif
+#endif
+            break;
+        case QImage::Format_RGB888:
+            frmt = ZXing::ImageFormat::RGB;
+            break;
+        case QImage::Format_RGBX8888:
+        case QImage::Format_RGBA8888:
+#if ZXING_VERSION_MAJOR <=2 && ZXING_VERSION_MINOR <= 2
+            frmt = ZXing::ImageFormat::RGBX;
+#else
+            frmt = ZXing::ImageFormat::RGBA;
+#endif
+            break;
+        case QImage::Format_Grayscale8:
+            frmt = ZXing::ImageFormat::Lum;
+            break;
+        default:
+            img.convertTo(QImage::Format_Grayscale8);
+            frmt = ZXing::ImageFormat::Lum;
+    }
 
 #if ZXING_VERSION_MAJOR == 1 && ZXING_VERSION_MINOR <= 2
 
@@ -867,8 +825,12 @@ QVariantList PQCScriptsImages::getZXingData(QString path) {
 #else
 
     // Read all bar codes
-    const ZXing::ReaderOptions hints = ZXing::ReaderOptions().setFormats(ZXing::BarcodeFormat::Any);
-    const std::vector<ZXing::Result> results = ZXing::ReadBarcodes({img.bits(), img.width(), img.height(), ZXing::ImageFormat::Lum}, hints);
+    auto hints = ZXing::ReaderOptions().setFormats(ZXing::BarcodeFormat::Any)
+                                                             .setTryHarder(true)
+                                                             .setTryInvert(true)
+                                                             .setTextMode(ZXing::TextMode::HRI)
+                                                             .setMaxNumberOfSymbols(10);
+    auto results = ZXing::ReadBarcodes({img.bits(), img.width(), img.height(), frmt, static_cast<int>(img.bytesPerLine())}, hints);
 
 #endif
 
@@ -1018,7 +980,10 @@ QString PQCScriptsImages::extractArchiveFileToTempLocation(QString path) {
                 dir.mkpath(info.absolutePath());
 
             // write buffer to file
-            file.open(QIODevice::WriteOnly);
+            if(!file.open(QIODevice::WriteOnly)) {
+                qWarning() << "Unable to extract file to temporary location.";
+                return "";
+            }
             QDataStream out(&file);   // we will serialize the data into the file
             out.writeRawData((const char*) buff,size);
             file.close();
@@ -1412,3 +1377,20 @@ double PQCScriptsImages::getPixelDensity() {
     return qApp->devicePixelRatio();
 
 }
+
+QString PQCScriptsImages::getNameFromMimetype(QString mimetype, QString filename) {
+
+    QMimeDatabase db;
+
+    QString val = db.mimeTypeForName(mimetype).comment();
+    if(val == "")
+        val = PQCImageFormats::get().getFormatName(PQCImageFormats::get().detectFormatId(filename));
+
+    return val;
+
+}
+
+QString PQCScriptsImages::getMimetypeForFile(QString path) {
+    QMimeDatabase db;
+    return db.mimeTypeForFile(path).name();
+}
